USE TEMPDB
GO

SELECT CAST('20080101' AS datetime) as DateOnly
,CAST('14:00:00.000' AS datetime) as TimeOnly
GO

-- Precision of the time data type
SELECT  CAST('14:13:12.1234567' AS time) AS [time]
,CAST('14:13:12.1234567' AS time(0) ) AS [time(0)]
,CAST('14:13:12.1234567' AS time(1) ) AS [time(1)]
,CAST('14:13:12.1234567' AS time(2) ) AS [time(2)]
,CAST('14:13:12.1234567' AS time(3) ) AS [time(3)]
,CAST('14:13:12.1234567' AS time(4) ) AS [time(4)]
,CAST('14:13:12.1234567' AS time(5) ) AS [time(5)]
,CAST('14:13:12.1234567' AS time(6) ) AS [time(6)]
,CAST('14:13:12.1234567' AS time(7) ) AS [time(7)]

-- Precision of the datetime2 data type
SELECT  CAST('2008-05-29 14:13:12.1234567' AS datetime2) AS [datetime2]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (0) ) AS [datetime2(0)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (1) ) AS [datetime2(1)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (2) ) AS [datetime2(2)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (3) ) AS [datetime2(3)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (4) ) AS [datetime2(4)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (5) ) AS [datetime2(5)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2 (6) ) AS [datetime2(6)]
,CAST('2008-05-29 14:13:12.1234567' AS datetime2(7) ) AS [datetime2(7)]


-- Defaulting to midnight if time portion not specified.
SELECT  CAST('2008-05-29' AS datetime2) AS [datetime2]

-- Datetimeoffsets
SELECT CAST('2008-05-29 16:28:12.1234567 +02:15' AS datetimeoffset(7)) AS 'datetimeoffset'
    ,CAST('2008-05-29T11:58:12.1234567-02:15' AS datetimeoffset(7)) AS  'datetimeoffset IS08601'
    ,CAST('2008-05-29T14:13:12.1234567Z' AS datetimeoffset(7)) AS  'datetimeoffset UTC'
   
-- Date Difference using new date parts  
DECLARE @datetime2 datetime2(7), @datetimeoffset   datetimeoffset   
SET @datetime2 = '2008-05-29T14:13:12.1234567'
SET @datetimeoffset  = '2008-05-29T14:13:12.1234567+02:15'

SELECT DATEPART ( microsecond, @datetime2 ) AS [Microsecond]
    ,DATEPART ( nanosecond, @datetime2 ) AS [Nanosecond]
	,DATEPART ( TZoffset, @datetimeoffset ) AS [TZoffset]
	,DATEPART ( ISO_WEEK, @datetime2 ) AS [ISO_Week]

SELECT DATEPART ( mcs, @datetime2 ) AS [Microsecond]
    ,DATEPART ( ns, @datetime2 ) AS [Nanosecond]
	,DATEPART ( tz, @datetimeoffset  ) AS [TZoffset]
	,DATEPART ( isowk, @datetime2 ) AS [ISO_Week]


DECLARE @datetime2 datetime2(7), @datetimeoffset   datetimeoffset   
SET @datetime2 = '2008-05-29T14:13:12.1234567'
SET @datetimeoffset  = '2008-05-29T14:13:12.1234567+02:15'

SELECT @datetime2,
  DATEADD ( microsecond, 1, @datetime2 ) AS [Microsecond]
    ,DATEADD ( nanosecond, 1, @datetime2 ) AS [Nanosecond]
GO
    
-- Date Difference using ISO_WEEK(not supported)
DECLARE @datetime2_1 datetime2(7), @datetime2_2 datetime2
SELECT @datetime2_1 = '2007-12-31T14:13:12.1234567',
 	 @datetime2_2 = '2008-01-06T14:13:12.1234578'

SELECT DATEDIFF( ISO_WEEK, @datetime2_1, @datetime2_2 ) AS [ISO Week Difference]
GO
/*
Msg 9806, Level 16, State 0, Line 6
The datepart iso_week is not supported by date function datediff.
*/

-- DAY, MONTH and YEAR support datetime2, datetimeoffset and date data types
DECLARE @datetime2 datetime2(7), @datetimeoffset datetimeoffset, @date date
SELECT @datetime2 = '2008-05-29T14:13:12.1234567'
 	 ,@datetimeoffset = '2008-05-29T14:13:12.1234567'
 	 ,@date = '2008-05-29'
 	 
SELECT DAY(@datetime2)
				,DAY(@datetimeoffset)
				,DAY(@date)
				,MONTH(@datetime2)
				,MONTH(@datetimeoffset)
				,MONTH(@date)
				,YEAR(@datetime2)
				,YEAR(@datetimeoffset)
				,YEAR(@date)
GO

-- High precision system functions
-- Using Current Time (GMT or BST)
SELECT SYSDATETIME() AS [SysDateTime]
		,SYSDATETIMEOFFSET() AS [SysDateTimeOffset]
		,SYSUTCDATETIME()AS [SysUTCDateTime]
GO
		
-- Change timezone to GMT -8:00 (Pacific Time)
SELECT SYSDATETIME() AS [SysDateTime]
		,SYSDATETIMEOFFSET() AS [SysDateTimeOffset]
		,SYSUTCDATETIME()AS [SysUTCDateTime]
GO
		
-- Change back to GMT
SELECT SYSDATETIMEOFFSET() as [Current Time]
			,SWITCHOFFSET(SYSDATETIMEOFFSET(), '-07:00' ) AS [Pacific Summer Time]
GO

DECLARE @datetimeoffset datetimeoffset
SET @datetimeoffset = SYSDATETIMEOFFSET() -- Current Time (GMT)

SELECT @datetimeoffset, TODATETIMEOFFSET(@datetimeoffset, '-07:00' ) AS [Pacific Time]
GO