USE TEMPDB
GO

SET NOCOUNT ON
GO

CREATE TABLE myData([Date] datetime,Area char(2),Product char(4),Amount int) 
GO

INSERT INTO myData([Date],Area,Product,Amount) 
SELECT '20080423','CA','Book', 150 
UNION ALL SELECT '20080429','NY','Pen', 70 
UNION ALL SELECT '20080503','CA','Pen', 500 
UNION ALL SELECT '20080504','CA','Book', 200 
UNION ALL SELECT '20080505','NY','Book', 270 
UNION ALL SELECT '20080505','CA','Pen', 400 
UNION ALL SELECT '20080505','NY','Pen', 300 
UNION ALL SELECT '20080508','NY','Book', 150
UNION ALL SELECT '20080508','CA','Book', 120
GO

CREATE VIEW MyMonthlyValues AS 
SELECT YEAR([Date]) AS Year,  MONTH([Date]) AS [Month], Area, Product, Amount 
FROM MyData 
GO

SELECT * FROM MyMonthlyValues
ORDER BY [Year], [Month], Area, Product
GO

SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY  [Year], [Month], Area, Product
/*
Year        Month       Area Product AmountTotal
----------- ----------- ---- ------- -----------
2008        4           CA   Book    150
2008        4           NY   Pen     70
2008        5           CA   Book    320
2008        5           CA   Pen     900
2008        5           NY   Book    420
2008        5           NY   Pen     300
*/

-- Same As
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY  GROUPING SETS ( ( [Year] , [Month], Area, Product ) )
GO

SELECT [Year], [Month], NULL AS Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY  [Year] , [Month], Product
UNION ALL
SELECT [Year], [Month], Area, NULL AS Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY  [Year] , [Month], Area
ORDER BY  [Year] , [Month], Area, Product
GO
/*
Year        Month       Area Product AmountTotal
----------- ----------- ---- ------- -----------
2008        4           NULL Book    150
2008        4           NULL Pen     70
2008        4           CA   NULL    150
2008        4           NY   NULL    70
2008        5           NULL Book    740
2008        5           NULL Pen     1200
2008        5           CA   NULL    1220
2008        5           NY   NULL    720
*/
-- Same as
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY GROUPING SETS ( ( [Year] , [Month], Area ), ([Year] , [Month], Product ))
ORDER BY  [Year] , [Month], Area, Product

-- Same As
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY CUBE ( [Year] , [Month], Area, Product )
 HAVING YEAR IS NOT NULL AND MONTH IS NOT NULL AND ( 
( Area IS NULL AND Product IS NOT NULL )
OR 
( Area IS NOT NULL AND Product IS NULL ) )
ORDER BY  [Year] , [Month], Area, Product
GO

-- ROLLUP query
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY ROLLUP ( [Year] , [Month], Area, Product )
ORDER BY  [Year] , [Month], Area, Product
/*
Year        Month       Area Product AmountTotal
----------- ----------- ---- ------- -----------
NULL        NULL        NULL NULL    2160
2008        NULL        NULL NULL    2160
2008        4           NULL NULL    220
2008        4           CA   NULL    150
2008        4           CA   Book    150
2008        4           NY   NULL    70
2008        4           NY   Pen     70
2008        5           NULL NULL    1940
2008        5           CA   NULL    1220
2008        5           CA   Book    320
2008        5           CA   Pen     900
2008        5           NY   NULL    720
2008        5           NY   Book    420
2008        5           NY   Pen     300
*/

-- Same as
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY GROUPING SETS ( ([Year], [Month], Area, Product)
    ,([Year], [Month],Area)
    ,([Year], [Month])
    ,([Year])
    ,() )
ORDER BY  [Year] , [Month], Area, Product

-- CUBE query
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY CUBE  ( [Year] , [Month], Area, Product )
ORDER BY  [Year] , [Month], Area, Product
/*
Year        Month       Area Product AmountTotal
----------- ----------- ---- ------- -----------
NULL        NULL        NULL NULL    2160
NULL        NULL        NULL Book    890
NULL        NULL        NULL Pen     1270
NULL        NULL        CA   NULL    1370
NULL        NULL        CA   Book    470
NULL        NULL        CA   Pen     900
NULL        NULL        NY   NULL    790
NULL        NULL        NY   Book    420
NULL        NULL        NY   Pen     370
NULL        4           NULL NULL    220
NULL        4           NULL Book    150
NULL        4           NULL Pen     70
NULL        4           CA   NULL    150
NULL        4           CA   Book    150
NULL        4           NY   NULL    70
NULL        4           NY   Pen     70
NULL        5           NULL NULL    1940
NULL        5           NULL Book    740
NULL        5           NULL Pen     1200
NULL        5           CA   NULL    1220
NULL        5           CA   Book    320
NULL        5           CA   Pen     900
NULL        5           NY   NULL    720
NULL        5           NY   Book    420
NULL        5           NY   Pen     300
2008        NULL        NULL NULL    2160
2008        NULL        NULL Book    890
2008        NULL        NULL Pen     1270
2008        NULL        CA   NULL    1370
2008        NULL        CA   Book    470
2008        NULL        CA   Pen     900
2008        NULL        NY   NULL    790
2008        NULL        NY   Book    420
2008        NULL        NY   Pen     370
2008        4           NULL NULL    220
2008        4           NULL Book    150
2008        4           NULL Pen     70
2008        4           CA   NULL    150
2008        4           CA   Book    150
2008        4           NY   NULL    70
2008        4           NY   Pen     70
2008        5           NULL NULL    1940
2008        5           NULL Book    740
2008        5           NULL Pen     1200
2008        5           CA   NULL    1220
2008        5           CA   Book    320
2008        5           CA   Pen     900
2008        5           NY   NULL    720
2008        5           NY   Book    420
2008        5           NY   Pen     300
*/

-- Same as
SELECT [Year], [Month], Area, Product,
			SUM(Amount) AS AmountTotal 
FROM MyMonthlyValues 
GROUP BY GROUPING SETS ( ([Year], [Month], Area, Product)
    ,([Year], [Month],Area)
    ,([Year], [Month],Product)
    ,([Year], Area, Product)
    ,([Month], Area, Product)
    ,([Year], [Month])
    ,([Year], Area)
    ,([Year], Product)
    ,([Month], Area)
    ,([Month], Product)
    ,(Area, Product)
    ,([Year])
    ,([Month])
    ,(Area)
    ,(Product)
    ,()  )
ORDER BY  [Year] , [Month], Area, Product
-- Clean up
DROP TABLE myData
GO

DROP VIEW MyMonthlyValues
GO
