USE TEMPDB
GO

CREATE TABLE Products  (
	[ProductID] [int] NOT NULL CONSTRAINT PK_Products PRIMARY KEY CLUSTERED,
	[Name] [nvarchar](100) NOT NULL,
	[ProductNumber] [nvarchar](25) NOT NULL,
	[Colour] [nvarchar](15) NULL )
GO

INSERT INTO Products  (	[ProductID], [Name], 	[ProductNumber],	[Colour] )
VALUES 
( 991, 'Mountain-500 Black, 44', 'BK-M18B-44', 'Black'),
( 992, 'Mountain-500 Black, 48', 'BK-M18B-48', 'Black'),
( 993, 'Mountain-500 Black, 52', 'BK-M18B-52', 'Black')
GO

-- Make sure Grid Mode is on!
SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products
GO

-- Create a load table
CREATE TABLE Products_load  (
	[ProductID] [int] NOT NULL,
	[Name] [nvarchar](100) NOT NULL,
	[ProductNumber] [nvarchar](25) NOT NULL,
	[Colour] [nvarchar](15) NULL )
GO

-- Updated product the "Mountain-500S"	IDs 991, 992 and 993
INSERT INTO Products_load  (	[ProductID], [Name], 	[ProductNumber],	[Colour] )
SELECT [ProductID], [Name], 	[ProductNo],	[Colour]
FROM ( VALUES 
( 990, 'Mountain-500S Black, 42', 'BK-M18B-42', 'Black'),
( 991, 'Mountain-500S Black, 44', 'BK-M18B-44', 'Black'),
( 992, 'Mountain-500S Black, 48', 'BK-M18B-48', 'Black'),
( 993, 'Mountain-500S Black, 52', 'BK-M18B-52', 'Black'),
( 997, 'Road-750 Black, 44', 'BK-R19B-44', 'Black'),
( 998, 'Road-750 Black, 48', 'BK-R19B-48', 'Black'),
( 999, 'Road-750 Black, 52', 'BK-R19B-52', 'Black')
) ld ( [ProductID], [Name], 	[ProductNo],	[Colour]  )
GO

-- New/Updated Data
SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load
GO

-- Just doing Updates on matches to productId
-- Always terminate with semi-colon!!
MERGE Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour] 
    ; 
GO

-- Updated Data
SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products
GO

-- Change the loaded data
UPDATE Products_load  
SET [Colour] = 'Noir'
WHERE [Colour] = 'Black'
GO
    
-- Doing Updates and Inserts on matches to productId
MERGE Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour]
WHEN NOT MATCHED
	THEN  INSERT  ( [ProductID], [Name], [ProductNumber], [Colour] )
			VALUES ( src.[ProductID], src.[Name], src.[ProductNumber], src.[Colour] ) ;
GO			

-- Updated/Inserted data
SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products
GO

-- Reset Load Data
TRUNCATE TABLE Products_load 
GO

INSERT INTO Products_load  (	[ProductID], [Name], 	[ProductNumber],	[Colour] )
VALUES 
( 978, 'Touring-3000 Blue, 44', 'BK-T18U-44', 'Blue'),
( 979, 'Touring-3000 Blue, 50', 'BK-T18U-50', 'Blue'),
( 980, 'Mountain-400-W Silver, 38', 'BK-M38S-38', 'Silver'),
( 981, 'Mountain-400-W Silver, 40', 'BK-M38S-40', 'Silver'),
( 982, 'Mountain-400-W Silver, 42', 'BK-M38S-42', 'Silver'),
( 983, 'Mountain-400-W Silver, 46', 'BK-M38S-46', 'Silver'),
( 984, 'Mountain-500S Silver, 40', 'BK-M18S-40', 'Silver'),
( 985, 'Mountain-500S Silver, 42', 'BK-M18S-42', 'Silver'),
( 986, 'Mountain-500S Silver, 44', 'BK-M18S-44', 'Silver'),
( 987, 'Mountain-500S Silver, 48', 'BK-M18S-48', 'Silver'),
( 988, 'Mountain-500S Silver, 52', 'BK-M18S-52', 'Silver'),
( 989, 'Mountain-500S Black, 40', 'BK-M18B-40', 'Black'),
( 990, 'Mountain-500S Black, 42', 'BK-M18B-42', 'Black'),
( 991, 'Mountain-500S Black, 44', 'BK-M18B-44', 'Black'),
( 992, 'Mountain-500S Black, 48', 'BK-M18B-48', 'Black'),
( 993, 'Mountain-500S Black, 52', 'BK-M18B-52', 'Black')
/* Road Bikes have been discontinued!
( 997, 'Road-750 Black, 44', 'BK-R19B-44', 'Black'),
( 998, 'Road-750 Black, 48', 'BK-R19B-48', 'Black'),
( 999, 'Road-750 Black, 52', 'BK-R19B-52', 'Black')
 */
GO
 
-- Doing Updates/Inserts and Deletes on matches to productId
MERGE Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour]
WHEN NOT MATCHED
	THEN  INSERT  ( [ProductID], [Name], [ProductNumber], [Colour] )
			VALUES ( src.[ProductID], src.[Name], src.[ProductNumber], src.[Colour] )
WHEN NOT MATCHED BY SOURCE THEN
    DELETE ;
GO		

-- Updated/Inserted/Deleted data
SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products
ORDER BY ProductID
GO

-- Change some data
DELETE FROM Products
WHERE ProductId < 981

UPDATE Products
SET Colour = 'gris'
WHERE Colour = 'Silver'
GO

-- Doing Updates/Inserts and Deletes on matches to productId with TOP and OUTPUT clauses
-- TOP applied to each WHEN after unqualifying rows are removed
MERGE TOP(5) Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour]
WHEN NOT MATCHED
	THEN  INSERT  ( [ProductID], [Name], [ProductNumber], [Colour] )
			VALUES ( src.[ProductID], src.[Name], src.[ProductNumber], src.[Colour] )
WHEN NOT MATCHED BY SOURCE THEN
    DELETE 
    OUTPUT $ACTION, COALESCE(inserted.ProductId, deleted.ProductId ) AS ProductId ;
GO		

SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products
ORDER BY ProductID
GO

INSERT INTO Products_load  (	[ProductID], [Name], 	[ProductNumber],	[Colour] )
VALUES 
( 1000, 'City Bike-3000 Red, 44', 'CB-T30U-44', 'Red'),
( 1001, 'City Bike-4000 Red, 50', 'BK-T40U-50', 'Red'),
( 1001, 'City Bike-4000 Red, 50', 'BK-T40U-50', 'Red')
;

-- Doing Updates/Inserts and Deletes on matches to productId with OUTPUT clauses
MERGE Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour]
WHEN NOT MATCHED
	THEN  INSERT  ( [ProductID], [Name], [ProductNumber], [Colour] )
			VALUES ( src.[ProductID], src.[Name], src.[ProductNumber], src.[Colour] )
WHEN NOT MATCHED BY SOURCE THEN
    DELETE 
    OUTPUT $ACTION, COALESCE(inserted.ProductId, deleted.ProductId ) AS ProductId ;
GO		

-- Doing Updates/Inserts and Deletes on matches to productId with OUTPUT clauses
-- When data is not distinct
MERGE Products AS p
USING 
	(SELECT [ProductID], [Name], [ProductNumber], [Colour] FROM Products_load ) as src
		ON (p.ProductID = src.ProductID)
WHEN MATCHED AND ( p.[Name] <> src.[Name] OR p.[ProductNumber] <> src.[ProductNumber] OR p.[Colour] <> src.[Colour] )
    THEN UPDATE SET p.[Name] = src.[Name], p.[ProductNumber] = src.[ProductNumber],  p.[Colour] = src.[Colour]
WHEN NOT MATCHED
	THEN  INSERT  ( [ProductID], [Name], [ProductNumber], [Colour] )
			VALUES ( src.[ProductID], src.[Name], src.[ProductNumber], src.[Colour] )
WHEN NOT MATCHED BY SOURCE THEN
    DELETE 
    OUTPUT $ACTION, COALESCE(inserted.ProductId, deleted.ProductId ) AS ProductId ;
GO		

/*
Msg 2627, Level 14, State 1, Line 2
Violation of PRIMARY KEY constraint 'PK_Products'. Cannot insert duplicate key in object 'dbo.Products'.
The statement has been terminated.
*/

-- Clean Up
DROP TABLE Products 
GO

DROP TABLE Products_load
GO
